/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2013, Digium, Inc.
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*!
 * \file
 * \brief PJSIP Channel Driver shared data structures
 */

#ifndef _CHAN_PJSIP_HEADER
#define _CHAN_PJSIP_HEADER

struct ast_sip_session_media;

/*!
 * \brief Transport information stored in transport_info datastore
 */
struct transport_info_data {
	/*! \brief The address that sent the request */
	pj_sockaddr remote_addr;
	/*! \brief Our address that received the request */
	pj_sockaddr local_addr;
};

/*!
 * \brief Positions of various media
 */
enum sip_session_media_position {
	/*! \brief First is audio */
	SIP_MEDIA_AUDIO = 0,
	/*! \brief Second is video */
	SIP_MEDIA_VIDEO,
#ifdef GRANDSTREAM_NETWORKS
	/* Add by bfdu for BFCP/FEC/FECC */
	/*! \brief third is application, like bfcp/fec/fecc */
	SIP_MEDIA_APPLICATION,
#endif
	/*! \brief Last is the size for media details */
	SIP_MEDIA_SIZE,
};

/*!
 * \brief The PJSIP channel driver pvt, stored in the \ref ast_sip_channel_pvt
 * data structure
 */
struct chan_pjsip_pvt {
	/*! \brief The available media sessions */
	struct ast_sip_session_media *media[SIP_MEDIA_SIZE];

#ifdef GRANDSTREAM_NETWORKS
	char *call_type;			/*!< Call type */
	char *mcb_request;			/*!< MCB request in INVITE */
	char *mcb_response;			/*!< MCB response in 200 OK */
	char *contact_user;			/*!< Contact URI Host User Part */
	int isConference;
#endif
};
#ifdef GRANDSTREAM_NETWORKS
typedef enum {
	GS_FEATURE_DISABLE      = 0,
	GS_FEATURE_ALLOW_CALLER = 0x01,
	GS_FEATURE_ALLOW_CALLEE = 0x10,
	GS_FEATURE_ALLOW_BOTH   = 0x11,
	GS_FEATURE_RIGHT_BUTT
} gs_feature_right;

typedef enum {
	GS_FEATURE_BLINDXFER  = 0,
	GS_FEATURE_ATXFER     = 1,
	GS_FEATURE_PARKCALL   = 2,
	GS_FEATURE_AUTOMIXMON = 3,
	GS_FEATURE_MAP_COUNT
} gs_feature_map_enum;


struct ast_gs_feature_map {
	char *f_name;
	char  f_code[12];
	int   f_right;     /* gs_feature_right */
};
#endif

#endif /* _CHAN_PJSIP_HEADER */
