#ifndef __DLOG_H_
#define __DLOG_H_

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

#define DEBUG

enum l_level
{
    L_EMERG,
    L_ALERT,
    L_CRIT,
    L_ERR,
    L_WARNING,
    L_NOTICE,
    L_INFO,
    L_DBG,
};
typedef enum l_level llevel_t;

extern llevel_t dlog_level;
extern int log_to_stderr;

int dlog(llevel_t level, const char *fmt, ...);

#ifdef DEBUG
#define EMERG(fmt, args...)         dlog(L_EMERG, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define ALERT(fmt, args...)         dlog(L_ALERT, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define CRIT(fmt, args...)          dlog(L_CRIT, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define ERR(fmt, args...)           dlog(L_ERR, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define WARNING(fmt, args...)       dlog(L_WARNING, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define NOTICE(fmt, args...)        dlog(L_NOTICE, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define INFO(fmt, args...)          dlog(L_INFO, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#define DBG(fmt, args...)           dlog(L_DBG, "%s-%d: "fmt, __FILE__, __LINE__, ##args)
#else
#define EMERG(fmt, args...)         dlog(L_EMERG, fmt, ##args)
#define ALERT(fmt, args...)         dlog(L_ALERT, fmt, ##args)
#define CRIT(fmt, args...)          dlog(L_CRIT, fmt, ##args)
#define ERR(fmt, args...)           dlog(L_ERR, fmt, ##args)
#define WARNING(fmt, args...)       dlog(L_WARNING, fmt, ##args)
#define NOTICE(fmt, args...)        dlog(L_NOTICE, fmt, ##args)
#define INFO(fmt, args...)          dlog(L_INFO, fmt, ##args)
#define DBG(fmt, args...)           dlog(L_DBG, fmt, ##args)
#endif

#endif

