/*
 * Vinetic-SVIP Codec control.
 *
 * Copyright (C) 2012 - 2014, Grandstream Networks, Inc.
 *
 * jwli@grandstream.com
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef __GXE_INIT_SVIP_H__
#define __GXE_INIT_SVIP_H__

#include <sys/types.h>
#include <netinet/in.h>

#include "ifx_types.h"
#include "svip_io.h"
#include "drv_tapi_io.h"
#include "lib_svip_rm.h"
#include "lib_svip.h"
#include "ifxos_print.h"
#include "linux/ifxos_linux_print.h"
#include "dlog.h"

#include "asterisk/svipmanager.h"

#define VETH_NAME "svip0"
#define ETH_NAME VETH_NAME
#define CODEC_UDPPORT0 61000

#define MAX_SVIP_CODEC_CHANNEL (MAX_VINETIC_DEV_NUM * MAX_VINETIC_CHAN_PER_DEV)
#define IS_INVALID_ENC_TYPE(x) ((((x) > IFX_TAPI_COD_TYPE_UNKNOWN && \
	(x) < IFX_TAPI_COD_TYPE_MAX && (x)!= 3 && (x)!= 4 && (x)!= 5))?0:1)

typedef enum {
	COD_ENC_START,
	COD_ENC_HOLD,
	COD_ENC_STOP,
	COD_DEC_START,
	COD_DEC_HOLD,
	COD_DEC_STOP,
} cod_ctrl;

struct svip_trans_formats {
	IFX_uint32_t srcfmt;
	IFX_uint32_t dstfmt;
};

typedef enum {
	JB_TYPE_FIXED = 0,
	JB_TYPE_ADAPTIVE,
	JB_TYPE_END,
} svip_jb_type_t;

typedef enum {
	JB_LEN_LOW = 0,
	JB_LEN_MEDIUM = 1,
	JB_LEN_HIGH = 3,
	JB_LEN_END,
} svip_jb_len_t;

typedef struct {
	svip_jb_type_t type;
	svip_jb_len_t length;
} svip_jb_setting_t;

typedef struct rtp_header *rtp_header_t;
typedef struct rtp_packet *rtp_packet_t;

struct rtp_packet {
	void* data;
	unsigned int data_len;
};

struct rtp_header {
	unsigned int csrc_count:4;    /* CSRC count */
	unsigned int extension:1;     /* header extension flag */
	unsigned int padding:1;       /* padding flag */
	unsigned int version:2;       /* protocol version */
	unsigned int payload_type:7;  /* payload type */
	unsigned int marker:1;        /* marker bit */
	unsigned short seq;            /* sequence number */
	unsigned int timestamp;        /* timestamp */
	unsigned int ssrc;             /* synchronization source */
	unsigned int csrc[1];          /* optional CSRC list */
};

typedef struct {
	int snd_dstport;
	int rcv_dstport;
	int snd_srcport;
	struct sockaddr_in svip_saddr;
	struct sockaddr_in rcv_saddr;
	int sock;
	rtp_packet_t rtp_pkt_snd;
	rtp_packet_t rtp_pkt_rcv;
	int rtp_ssrc;
	int rtp_pt;
	unsigned int seq_num;
	unsigned int timestamp;
	unsigned int timestamp_increse;
	int m_bit;
	int rtpinf;
} svip_rtp_sess_t;

/*
* Current protocol version.
*/
enum {
	RTP_VERSION = 2,
	RTP_MTU = 2048
};

enum {
	RTP_HEADER_LEN = 12
};

llevel_t vinetic_set_loglevel(int level);

llevel_t vinetic_get_loglevel(void);

IFX_return_t vinetic_set_jb_typelen(svip_jb_setting_t jb_setting);

IFX_return_t vinetic_get_jb_typelen(svip_jb_setting_t *jb_setting);

IFX_return_t vinetic_check_codec_type(int fd, IFX_uint16_t dev,
	IFX_TAPI_COD_TYPE_t type);

IFX_int32_t vinetic_check_codec_num(int fd, IFX_uint16_t dev);

IFX_return_t vinetic_get_jb_statistic(int fd, IFX_uint16_t dev,
	IFX_uint16_t ch, IFX_TAPI_JB_STATISTICS_t *jbstat,
	IFX_TAPI_PKT_RTCP_STATISTICS_t *rtcpstat);

IFX_return_t vinetic_codec_cfg(int fd, IFX_uint16_t dev, IFX_uint16_t ch,
	IFX_TAPI_COD_TYPE_t enctype, IFX_uint16_t framelen,
	IFX_uint16_t udpdst, IFX_uint16_t udprcv, IFX_uint16_t udpsrc);

IFX_int16_t vinetic_svip_get_cod_rcvport(IFX_uint16_t dev,
	IFX_uint16_t ch);

IFX_return_t vinetic_map_cod_ru(int fd, IFX_uint16_t dev, IFX_uint16_t ch,
	IFX_uint16_t dstch);

IFX_return_t vinetic_unmap_cod_ru(int fd, IFX_uint16_t dev,
	IFX_uint16_t ch,
	IFX_uint16_t dstch);

IFX_return_t vinetic_cod_ctrl(int fd, IFX_uint16_t dev, IFX_uint16_t ch,
	cod_ctrl cctrl);

IFX_return_t vinetic_svip_codec_stop(int fd, IFX_uint16_t dev,
	IFX_uint16_t encch, IFX_uint16_t decch);

IFX_return_t vinetic_svip_devstart(int fd, IFX_uint16_t dev);

IFX_return_t vinetic_svip_devstop(int fd, IFX_uint16_t dev);

IFX_return_t vinetic_svip_pkt_flush(int fd, IFX_uint16_t dev,
	IFX_uint16_t ch);

IFX_return_t svip_user_rtp_init(IFX_uint16_t dev, IFX_uint16_t send_dport,
	IFX_uint32_t rtp_payload_len, IFX_uint8_t rtp_pt, IFX_uint32_t ts_inc,
	svip_rtp_sess_t * rtp_sess);

int svip_user_rtp_release(int sock ,rtp_packet_t *pkt_snd,
	rtp_packet_t *pkt_rcv);

unsigned char rtp_packet_get_padding(rtp_packet_t packet);

unsigned char rtp_packet_get_csrc_count(rtp_packet_t packet);

void *rtp_packet_get_payload(rtp_packet_t packet);

unsigned int rtp_packet_get_payload_len(rtp_packet_t packet);

unsigned int rtp_packet_get_timestamp(rtp_packet_t packet);

unsigned short rtp_packet_get_seqnum(rtp_packet_t packet);

void rtp_packet_set_version(rtp_packet_t packet, unsigned char version);

void rtp_packet_set_padding(rtp_packet_t packet, unsigned char padding);

void rtp_packet_set_extension(rtp_packet_t packet, unsigned char extension);

void rtp_packet_set_csrc_count(rtp_packet_t packet, unsigned char csrc_count);

void rtp_packet_set_marker(rtp_packet_t packet, unsigned char marker);

void rtp_packet_set_payload_type(rtp_packet_t packet, unsigned char payload_type);

void rtp_packet_set_seq(rtp_packet_t packet, unsigned int seq);

void rtp_packet_set_timestamp(rtp_packet_t packet, unsigned int timestamp);

void rtp_packet_set_ssrc(rtp_packet_t packet, unsigned int ssrc);

int rtp_packet_send(int sock, rtp_packet_t packet, struct sockaddr_in *toaddr);

#endif
