#ifdef GRANDSTREAM_NETWORKS
/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2014-2015, Junkai Lou <jklou@grandstream.cn>
 *
 * func_recordcheck.c adapted from the recordcheck for RECORD_CHECK 
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Functions for interaction with the Asterisk database
 *
 * \author Junkai Lou <jklou@grandstream.cn>
 *
 * \ingroup functions
 */

/*** MODULEINFO
	<support_level>core</support_level>
 ***/

#include "asterisk.h"

/* BEGIN:  add */
#undef AST_MODULE
#define AST_MODULE "database"
/*   END:  add */

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 328209 $")

#include <regex.h>

#include "asterisk/module.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"
#include "asterisk/astdb.h"

/*** DOCUMENTATION
	<function name="RECODE_CHECK" language="en_US">
		<synopsis>
			Read from or write to the Asterisk database.
		</synopsis>
		<syntax argsep="/">
			<parameter name="userlist" required="true" />
		</syntax>
		<description>
			<para>This function will check if the dialog will be recorded
			function.</para>
		</description>
		<see-also>
		</see-also>
	</function>
 ***/

static int recordcheck_function_read(struct ast_channel *chan, const char *cmd,
			char *data, char *buf, size_t len)
{
	char* res = NULL;

	char varname_tmp[256] = {0};
	char family[64] = {0};    
	char key[64] = {0};

	strcpy(varname_tmp, data);
	char *curfieldptr = varname_tmp;
	while( (res = strsep(&curfieldptr,"-")))
    {
		memset(family, 0, sizeof(family));
		snprintf(family, sizeof(family), "AAUSER/%s", res);
		snprintf(key, sizeof(key), "AUTORECORD");
		if (ast_ucm_db_get(family, key, buf, len - 1)) {
			ast_debug(1, "DB: %s/%s not found in database.\n", family, key);
		} else {
			if (strcasecmp(buf, "yes") == 0) {
				pbx_builtin_setvar_helper(chan, "REC_ENABLE", buf);
				break;
			} else {
				continue;
			}
		}
		memset(family, 0, sizeof(family));
		snprintf(family, sizeof(family), "GROUP/%s", res);
		snprintf(key, sizeof(key), "AUTORECORD");
		if (ast_ucm_db_get(family, key, buf, len - 1)) {
			ast_debug(1, "DB: %s/%s not found in database.\n", family, key);
		} else {
			if (strcasecmp(buf, "yes") == 0) {
				pbx_builtin_setvar_helper(chan, "REC_ENABLE", buf);
				break;
			} else {
				continue;
				}
		}
		memset(family, 0, sizeof(family));
		snprintf(family, sizeof(family), "QUEUE/%s", res);
		snprintf(key, sizeof(key), "AUTORECORD");
		if (ast_ucm_db_get(family, key, buf, len - 1)) {
			ast_debug(1, "DB: %s/%s not found in database.\n", family, key);
		} else {
			if (strcasecmp(buf, "yes") == 0) {
				pbx_builtin_setvar_helper(chan, "REC_ENABLE", buf);
				break;
			} else {
				continue;
			}
		}
	}
	return 0;
}


static struct ast_custom_function recordcheck_function = {
	.name = "RECORD_CHECK",
	.read = recordcheck_function_read,
};

static int unload_module(void)
{
	int res = 0;

	res |= ast_custom_function_unregister(&recordcheck_function);

	return res;
}

static int load_module(void)
{
	int res = 0;

	res |= ast_custom_function_register(&recordcheck_function);

	return res;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Record check related dialplan functions");
#endif

