#ifdef GRANDSTREAM_NETWORKS
/*********************************************
 * File Name : bfcp.h
 * Author: bfdu
 * Date:   2014.02.09
 * Description: for bfcp
 * Changes: Initial
********************************************/

#ifndef _ASTERISK_BFCP_H
#define _ASTERISK_BFCP_H

#include "asterisk/network.h"
#include "asterisk/frame.h"
#include "asterisk/io.h"
#include "asterisk/sched.h"
#include "asterisk/channel.h"
#include "asterisk/netsock2.h"
#include "asterisk/tcptls.h"
#include "asterisk/pbx.h"

#ifndef FALSE
#define FALSE    0
#endif

#ifndef TRUE
#define TRUE     1
#endif

#define BFCP_MAX_SETUP_SIZE 		16
#define BFCP_MAX_FLOORCTRL_SIZE 	8
#define BFCP_MAX_CONNECTION_SIZE 	16
#define BFCP_MAX_FINGERPRINT_SIZE 	64
#define BFCP_MAX_FLOORID_SIZE 		64
#define BFCP_MAX_CONTENT_SIZE		8
#define BFCP_MAX_DATAGRAM_LIMIT     1400
#define BFCP_XMIT_ERROR     		-2
#define DEFAULT_BFCP_AUTHLIMIT      100
#define DEFAULT_BFCP_AUTHTIMEOUT    30

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

struct ast_bfcp {
	int fd;
	char resp;
	struct ast_frame f;
	unsigned char rawdata[8192];
	int nat;
	int flags;
	struct ast_sockaddr us;
	struct ast_sockaddr them;
	int *ioid;
	struct sched_context *sched;
	struct io_context *io;
	void *data;
	char *tag;
};

struct ast_bfcp_tcp {
	struct ast_tcptls_session_args bfcp_tcp_desc;
	struct ast_tcptls_session_instance *tcptls_session;
	struct ast_bfcp_tcp *peer_bfcp_tcp;
	struct ast_sockaddr remote_address;
	struct ast_sockaddr peer_remote_address;
	int	destoryed_tcptls_session;
};

/*! ao2 object used for the FILE stream fopencookie()/funopen() cookie. */
struct ast_tcptls_stream {
	/*! SSL state if not NULL */
	SSL *ssl;
	/*!
	 * \brief Start time from when an I/O sequence must complete
	 * by struct ast_tcptls_stream.timeout.
	 *
	 * \note If struct ast_tcptls_stream.start.tv_sec is zero then
	 * start time is the current I/O request.
	 */
	struct timeval start;
	/*!
	 * \brief The socket returned by accept().
	 *
	 * \note Set to -1 if the stream is closed.
	 */
	int fd;
	/*!
	 * \brief Timeout in ms relative to struct ast_tcptls_stream.start
	 * to wait for an event on struct ast_tcptls_stream.fd.
	 *
	 * \note Set to -1 to disable timeout.
	 * \note The socket needs to be set to non-blocking for the timeout
	 * feature to work correctly.
	 */
	int timeout;
	/*! TRUE if stream can exclusively wait for fd input. */
	int exclusive_input;
};

struct ast_bfcp_para {
	unsigned int has_bfcp;			/* for UDP BFCP flag */
	unsigned int has_tcp_bfcp;		/* for TCP BFCP flag */
	int confid;
	int userid;
	char floorid[BFCP_MAX_FLOORID_SIZE];
	char floorctrl[BFCP_MAX_FLOORCTRL_SIZE];
	char setup[BFCP_MAX_SETUP_SIZE];
	char connection[BFCP_MAX_CONNECTION_SIZE];
	char fingerprint[BFCP_MAX_FINGERPRINT_SIZE];
};

void ast_bfcp_setnat(struct ast_bfcp *bfcp, int nat);
int ast_bfcp_fd(const struct ast_bfcp *bfcp);
void ast_bfcp_set_peer(struct ast_bfcp *bfcp, const struct ast_sockaddr *them);
void ast_bfcp_get_us(const struct ast_bfcp *bfcp, struct ast_sockaddr *us);
void ast_bfcp_stop(struct ast_bfcp *bfcp);
void ast_bfcp_destroy(struct ast_bfcp *bfcp);
struct ast_bfcp *ast_bfcp_new_with_bindaddr(const char *bfcp_protocol, struct sched_context *sched, struct io_context *io, struct ast_sockaddr *addr);
struct ast_frame *ast_bfcp_read(struct ast_bfcp *bfcp);
int ast_bfcp_write(struct ast_bfcp *s, struct ast_frame *f);
int ast_bfcp_setqos(struct ast_bfcp *bfcp, unsigned int tos, unsigned int cos);
int ast_bfcp_get_and_cmp_requested_target_address(struct ast_bfcp *bfcp, struct ast_sockaddr *address);

struct ast_bfcp_tcp *ast_tcp_bfcp_new_with_bindaddr(struct ast_sockaddr *addr);
void ast_bfcp_tcptls_server_start(struct ast_bfcp_tcp *bfcp_tcp);
void *handle_bfcp_tcptls_connection(void *data);
void *ast_bfcp_tcptls_server_root(void *data);
void bfcp_init_tcp_desc(struct ast_tcptls_session_args *bfcp_tcp_desc);
struct ast_tcptls_session_instance *ast_bfcp_tcptls_client_start(struct ast_bfcp_tcp *bfcp_tcp);
struct ast_tcptls_session_instance *ast_bfcp_tcptls_client_create(struct ast_tcptls_session_args *desc);
void bfcp_tcptls_client_args_destructor(void *obj);
struct ast_bfcp_tcp *create_peer_bfcp_tcp(struct ast_sockaddr *addr);
int bfcp_check_authtimeout(time_t start);
int bfcp_prepare_socket(struct ast_bfcp_tcp *bfcp_tcp, struct ast_bfcp_tcp *peer_bfcp_tcp);
void *_bfcp_tcp_helper_thread(struct ast_bfcp_tcp *bfcp_tcp);
void *bfcp_tcp_worker_fn(void *data);
HOOK_T ast_bfcp_tcptls_server_read(struct ast_tcptls_session_instance *tcptls_session, void *buf, size_t count);
HOOK_T ast_bfcp_tcptls_server_write(struct ast_tcptls_session_instance *tcptls_session, const void *buf, size_t count);
void ast_bfcp_tcp_get_us(const struct ast_bfcp_tcp *bfcp_tcp, struct ast_sockaddr *us);
void ast_bfcp_tcp_set_peer(struct ast_bfcp_tcp *bfcp_tcp, const struct ast_sockaddr *them);
void ast_bfcp_tcp_set_peer_remote(struct ast_bfcp_tcp *bfcp_tcp, const struct ast_sockaddr *them);
int ast_bfcp_tcp_get_and_cmp_requested_target_address(struct ast_bfcp_tcp *bfcp_tcp, struct ast_sockaddr *address);
void ast_bfcp_tcptls_server_stop(struct ast_bfcp_tcp *bfcp_tcp);
void ast_bfcp_tcp_destroy(struct ast_bfcp_tcp *bfcp_tcp);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif

#endif

