/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2014, Digium, Inc.
 *
 * Thammanoon Kawinfruangfukul <thammanoon@grandstream.com>
 * Weiling Liao <weiling.liao@grandstream.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

#ifndef _PJSIP_CEI_H
#define _PJSIP_CEI_H

#define CEI_TYPE "application"
#define CEI_SUBTYPE "conference-info+xml"
#define CEI_DATASTORE "CEI datastore"

#define CEI_NOTIFY_EVENT_JOIN "cei_ConfbridgeJoin"
#define CEI_NOTIFY_EVENT_LEAVE "cei_ConfbridgeLeave"
#define CEI_NOTIFY_EVENT_MUTE "cei_ConfbridgeMute"
#define CEI_NOTIFY_EVENT_LOCK "cei_ConfbridgeLock"
#define CEI_NOTIFY_EVENT_EXT_MCB "cei_ConfbridgeExtMCB"
#define CEI_NOTIFY_EVENT_INT_MCB "cei_ConfbridgeIntMCB"
#define CEI_NOTIFY_EVENT_MCB_LEAVE "cei_ConfbridgeMCBLeave"

#define CEI_USER_ROLE_CONFERENCE "Multi-conference Bridge"

#define CEI_MCB_TYPE_NONE "none"
#define CEI_MCB_TYPE_EXTERNAL "external"
#define CEI_MCB_TYPE_INTERNAL "local"

#define CEI_MCB_DIR_OUTGOING "outgoing"
#define CEI_MCB_DIR_INCOMING "incoming"

#define CEI_SUBSCRIPTION_BUCKETS 100
#define STASIS_BUCKETS 1
#define MAX_NOTIFY_USERS_DISPLAY 100

/* ENUMERATORS */
//Enumerator for the CEI event type passed in the ast_event
enum cei_event_type {
	CEI_EVENT_JOIN,
	CEI_EVENT_PART,
	CEI_EVENT_TOGGLE_MUTE,
	CEI_EVENT_TOGGLE_LOCK,
	CEI_EVENT_MCB_CONF_JOIN,
	CEI_EVENT_MCB_CONF_PART,
	CEI_EVENT_MCB_USER_JOIN,
	CEI_EVENT_MCB_USER_PART,
	CEI_EVENT_MCB_USER_TOGGLE_MUTE,
	CEI_EVENT_UNKNOWN,
};

enum cei_notify_type {
	CEI_NOTIFY_UNKNOWN,
	CEI_NOTIFY_FULL,
	CEI_NOTIFY_PARTIAl
};

enum cei_user_type {
	CEI_TYPE_UNKNOWN,
	CEI_TYPE_NORMAL_USER,
	CEI_TYPE_EXTERNAL_MCB,
	CEI_TYPE_INTERNAL_MCB
};

enum cei_conference_type {
	CEI_CONFERENCE_LOCAL,
	CEI_CONFERENCE_REMOTE
};

enum cei_user_role {
	CEI_ROLE_UNKNOWN,
	CEI_ROLE_USER_NORMAL,
	CEI_ROLE_USER_INVITE,
	CEI_ROLE_ADMIN,
	CEI_ROLE_CONFERENCE
};

enum cei_user_status {
	CEI_STATUS_UNKNOWN,
	CEI_STATUS_CONNECTED,
	CEI_STATUS_DISCONNECTED,
	CEI_STATUS_ONHOLD,
	CEI_STATUS_MUTEVIAFOCUS,
	CEI_STATUS_PENDING,
	CEI_STATUS_ALERTING,
	CEI_STATUS_DIALINGIN,
	CEI_STATUS_DIALINGOUT,
	CEI_STATUS_DISCONNECTING
};
/* END ENUMERATORS */

#endif /* _PJSIP_CEI_H */
