#ifdef GRANDSTREAM_NETWORKS
/*********************************************
 * File Name : bfcp.c
 * Author: bfdu
 * Date:   2014.02.09
 * Description: for bfcp
 * Changes: Initial
********************************************/
#define AST_MODULE_LOG "core"

#include "asterisk.h"
ASTERISK_FILE_VERSION(__FILE__, "$Revision: 123344 $")
#include <sys/time.h>
#include <signal.h>
#include <fcntl.h>

#include "asterisk/bfcp.h"
#include "asterisk/frame.h"
#include "asterisk/channel.h"
#include "asterisk/config.h"
#include "asterisk/lock.h"
#include "asterisk/utils.h"
#include "asterisk/netsock.h"
#include "asterisk/cli.h"
#include "asterisk/astobj2.h"
#include "asterisk/rtp_engine.h"

//static int bfcpstart = 50000;
//static int bfcpend = 52999;
static int bfcp_use_even_ports = TRUE;
#ifdef SO_NO_CHECK
static int nochecksums = 0;
#endif

//static int bfcp_tcp_start = 53000;
//static int bfcp_tcp_end = 55999;
int bfcp_unauth_sessions = 0;
int bfcp_authlimit = DEFAULT_BFCP_AUTHLIMIT;
int bfcp_authtimeout = DEFAULT_BFCP_AUTHTIMEOUT;

void ast_bfcp_setnat(struct ast_bfcp *bfcp, int nat)
{
	if (NULL == bfcp) {
		ast_log(LOG_WARNING, "bfcp is NULL, set nat failed");
		return;
	}
	bfcp->nat = nat;
}

int ast_bfcp_fd(const struct ast_bfcp *bfcp)
{
	if (NULL == bfcp) {
		ast_log(LOG_WARNING, "bfcp is NULL, get fd failed");
		return -1;
	}
	return bfcp->fd;
}

void ast_bfcp_set_peer(struct ast_bfcp *bfcp, const struct ast_sockaddr *them)
{
	if (NULL == bfcp || NULL == them) {
		ast_log(LOG_WARNING, "bfcp is NULL, set peer failed");
		return;
	}
	ast_sockaddr_copy(&bfcp->them, them);
}

void ast_bfcp_get_us(const struct ast_bfcp *bfcp, struct ast_sockaddr *us)
{
	if (NULL == bfcp || NULL == us) {
		ast_log(LOG_WARNING, "bfcp is NULL, get us failed");
		return;
	}
	ast_sockaddr_copy(us, &bfcp->us);
}

void ast_bfcp_stop(struct ast_bfcp *bfcp)
{
	if (NULL == bfcp) {
		ast_log(LOG_WARNING, "bfcp is NULL, stop failed");
		return;
	}
	ast_sockaddr_setnull(&bfcp->them);
}

void ast_bfcp_destroy(struct ast_bfcp *bfcp)
{
	if (NULL == bfcp) {
		ast_log(LOG_WARNING, "bfcp is NULL, destory failed");
		return;
	}
	if (bfcp->ioid)
		ast_io_remove(bfcp->io, bfcp->ioid);
	if (bfcp->fd > -1)
		close(bfcp->fd);
	if (bfcp->tag)
		ast_free(bfcp->tag);
	ast_free(bfcp);
}
struct ast_bfcp *ast_bfcp_new_with_bindaddr(const char *bfcp_protocol, struct sched_context *sched, struct io_context *io, struct ast_sockaddr *addr)
{
	struct ast_bfcp *bfcp;
	int x;
	int startplace;
	long int flags;
	int bprotocol = SOCK_DGRAM;
	int cnt = 0;

	ast_log(LOG_DEBUG, "ast_bfcp_new_with_bindaddr begin\n");

	int bfcpstart = ast_get_bfcp_start();
	int bfcpend = ast_get_bfcp_end();
	if (NULL == bfcp_protocol) {
		ast_log(LOG_WARNING, "bfcp_protocol is NULL");
		return NULL;
	}

	if (!(bfcp = ast_calloc(1, sizeof(*bfcp)))) {
		ast_log(LOG_WARNING, "Unable to alloc bfcp\n");
		return NULL;
	}

	if (!strcmp(bfcp_protocol, "TCP") || !strcmp(bfcp_protocol, "TCP/TLS")) {
		bprotocol = SOCK_STREAM;
	} else {
		bprotocol = SOCK_DGRAM;
	}
	ast_log(LOG_DEBUG, "ast_bfcp_new_with_bindaddr protocol is %d\n", bprotocol);
	/* todo, we use SOCK_DGRAM to creat socket now, later will support TCP*/
	if ((bfcp->fd = socket(ast_sockaddr_is_ipv6(addr) ?
					AF_INET6 : AF_INET, SOCK_DGRAM, 0)) < 0) {
		ast_free(bfcp);
		ast_log(LOG_WARNING, "Unable to allocate socket: %s\n", strerror(errno));
		return NULL;
	}
	flags = fcntl(bfcp->fd, F_GETFL);
	fcntl(bfcp->fd, F_SETFL, flags | O_NONBLOCK);
#ifdef SO_NO_CHECK
	if (nochecksums)
		setsockopt(bfcp->fd, SOL_SOCKET, SO_NO_CHECK, &nochecksums, sizeof(nochecksums));
#endif
	/* Find us a place */
	x = (bfcpstart == bfcpend) ? bfcpstart : (ast_random() % (bfcpend - bfcpstart)) + bfcpstart;
	if (bfcp_use_even_ports && (x & 1)) {
		++x;
	}
	startplace = x;
	for (;;) {
		ast_sockaddr_copy(&bfcp->us, addr);
		ast_sockaddr_set_port(&bfcp->us, x);
		if (ast_bind(bfcp->fd, &bfcp->us) == 0) {
			break;
		}
		if (errno != EADDRINUSE) {
			ast_log(LOG_WARNING, "Unexpected bfcp bind error: %s\n", strerror(errno));
			close(bfcp->fd);
			ast_free(bfcp);
			return NULL;
		}
		if (bfcp_use_even_ports) {
			x += 2;
			cnt++;
		} else {
			++x;
			cnt++;
		}
		if (x > bfcpend)
			x = bfcpstart;
		if (x == startplace) {
			ast_log(LOG_WARNING, "No bfcp ports remaining\n");
			close(bfcp->fd);
			ast_free(bfcp);
			return NULL;
		}
		if (cnt > 10)
		{
			ast_log(LOG_ERROR, "Could't allocate a port for bfcp\n");
			return NULL;
		}
	}
	return bfcp;
}

struct ast_frame *ast_bfcp_read(struct ast_bfcp *bfcp)
{
	int res;
	struct ast_sockaddr addr;
	uint8_t *buf;

	if (NULL == bfcp) {
		ast_log(LOG_WARNING, "ast_bfcp_read bfcp is NULL");
		return &ast_null_frame;
	}

	buf = bfcp->rawdata;

	/* Cache where the header will go */
	res = ast_recvfrom(bfcp->fd,
			buf,
			sizeof(bfcp->rawdata),
			0,
			&addr);
	if (res < 0) {
		if (errno != EAGAIN) {
			ast_log(LOG_WARNING, "bfcp : read error: %s\n", strerror(errno));
		}
		ast_assert(errno != EBADF);
		return &ast_null_frame;
	}

	ast_log(LOG_DEBUG, "ast_bfcp_read from %s", ast_sockaddr_stringify(&bfcp->them));

	/* Ignore if the other side hasn't been given an address yet. */
	if (ast_sockaddr_isnull(&bfcp->them)) {
		return &ast_null_frame;
	}

	if (bfcp->nat) {
		/* Send to whoever sent to us */
		if (ast_sockaddr_cmp(&bfcp->them, &addr)) {
			ast_sockaddr_copy(&bfcp->them, &addr);
			ast_debug(1, "bfcp : NAT, Using address %s\n", ast_sockaddr_stringify(&bfcp->them));
		}
	}

	bfcp->f.frametype = AST_FRAME_BFCP;
	bfcp->f.datalen = res;
	bfcp->f.data.ptr = buf;
	bfcp->f.offset = 0;
	bfcp->f.src = "BFCP";

	return &bfcp->f;
}

int ast_bfcp_write(struct ast_bfcp *s, struct ast_frame *f)
{
	unsigned int len = f->datalen;
	int res;
	/* if no max datagram size is provided, use default value */
	const int bufsize = BFCP_MAX_DATAGRAM_LIMIT;
	uint8_t buf[bufsize];

	memset(buf, 0, sizeof(buf));

	ast_log(LOG_DEBUG, "ast_bfcp_write begin send to %s", ast_sockaddr_stringify(&s->them));

	/* If we have no peer, return immediately */
	if (ast_sockaddr_isnull(&s->them)) {
		return 0;
	}

	/* If there is no data length, return immediately */
	if (f->datalen == 0)
		return 0;

	if (f->frametype != AST_FRAME_BFCP) {
		ast_log(LOG_WARNING, "BFCP can only send BFCP data.\n");
		return -1;
	}

	if (len > bufsize) {
		ast_log(LOG_WARNING, "bufsize is less than data len");
		return 0;
	}

	memcpy(buf, f->data.ptr, len);

	if ((signed int) len > 0 && !ast_sockaddr_isnull(&s->them)) {
		if ((res = ast_sendto(s->fd, buf, len, 0, &s->them)) < 0) {
			ast_log(LOG_NOTICE, "BFCP : Transmission error to %s: %s\n",
				ast_sockaddr_stringify(&s->them), strerror(errno));
		}
	}

	return 0;
}

int ast_bfcp_setqos(struct ast_bfcp *bfcp, unsigned int tos, unsigned int cos)
{
	ast_log(LOG_DEBUG, "ast_bfcp_setqos tos:%d, cos:%d", tos, cos);
	return ast_set_qos(bfcp->fd, tos, cos, "BFCP");
}

int ast_bfcp_get_and_cmp_requested_target_address(struct ast_bfcp *bfcp,
		struct ast_sockaddr *address)
{
	if (!bfcp || !address) {
		ast_log(LOG_WARNING, "bfcp cmp requested target address, param is null");
		return 0;
	}
	if (ast_sockaddr_cmp(address, &bfcp->them) != 0) {
		ast_sockaddr_copy(address, &bfcp->them);
		return 1;
	}

	return 0;
}

int ast_bfcp_tcp_get_and_cmp_requested_target_address(struct ast_bfcp_tcp *bfcp_tcp,
		struct ast_sockaddr *address)
{
	if (!bfcp_tcp || !address) {
		ast_log(LOG_WARNING, "bfcp cmp requested target address, param is null");
		return 0;
	}
	if (ast_sockaddr_cmp(address, &bfcp_tcp->bfcp_tcp_desc.remote_address) != 0) {
		ast_sockaddr_copy(address, &bfcp_tcp->bfcp_tcp_desc.remote_address);
		return 1;
	}

	return 0;
}

void ast_bfcp_tcp_get_us(const struct ast_bfcp_tcp *bfcp_tcp, struct ast_sockaddr *us)
{
	if (NULL == bfcp_tcp || NULL == us) {
		ast_log(LOG_WARNING, "bfcp is NULL, get us failed");
		return;
	}
	ast_sockaddr_copy(us, &bfcp_tcp->bfcp_tcp_desc.local_address);
}

void ast_bfcp_tcp_set_peer(struct ast_bfcp_tcp *bfcp_tcp, const struct ast_sockaddr *them)
{
	if (NULL == bfcp_tcp || NULL == them) {
		ast_log(LOG_WARNING, "bfcp is NULL, set peer failed");
		return;
	}
	ast_sockaddr_copy(&bfcp_tcp->bfcp_tcp_desc.remote_address, them);
	ast_sockaddr_copy(&bfcp_tcp->remote_address, them);
}

void ast_bfcp_tcp_set_peer_remote(struct ast_bfcp_tcp *bfcp_tcp, const struct ast_sockaddr *them)
{
	if (NULL == bfcp_tcp || NULL == them) {
		ast_log(LOG_WARNING, "bfcp is NULL, set peer failed");
		return;
	}
	ast_sockaddr_copy(&bfcp_tcp->peer_remote_address, them);
}

static void bfcp_session_instance_destructor(void *obj)
{
	struct ast_tcptls_session_instance *i = obj;
	ast_log(LOG_DEBUG, "bfcp_session_instance_destructor begin");
	if (i->stream_cookie) {
		ast_log(LOG_DEBUG, "bfcp_session_instance_destructor Destroying stream_cookie");
		ao2_t_ref(i->stream_cookie, -1, "Destroying tcptls session instance");
		i->stream_cookie = NULL;
	}
	ast_free(i->overflow_buf);
	ao2_cleanup(i->private_data);
}

static HOOK_T bfcp_tcptls_stream_read(void *cookie, char *buf, LEN_T size)
{
	struct ast_tcptls_stream *stream = cookie;
	struct timeval start;
	int ms;
	int res;

	if (!size) {
		/* You asked for no data you got no data. */
		return 0;
	}

	if (!stream || stream->fd == -1) {
		errno = EBADF;
		return -1;
	}

	if (stream->start.tv_sec) {
		start = stream->start;
	} else {
		start = ast_tvnow();
	}

	for (;;) {
		res = read(stream->fd, buf, size);
		if (0 <= res || !stream->exclusive_input) {
			/* Got data or we cannot wait for it. */
			return res;
		}
		if (errno != EINTR && errno != EAGAIN) {
			/* Not a retryable error. */
			ast_debug(1, "TCP socket error reading data: %s\n",
				strerror(errno));
			return -1;
		}
		ms = ast_remaining_ms(start, stream->timeout);
		if (!ms) {
			/* Report EOF for a timeout */
			ast_debug(1, "TCP timeout reading data\n");
			return 0;
		}
		ast_wait_for_input(stream->fd, ms);
	}
}

static HOOK_T bfcp_tcptls_stream_write(void *cookie, const char *buf, LEN_T size)
{
	ast_log(LOG_DEBUG, "bfcp_tcptls_stream_write date len:%d", size);
	struct ast_tcptls_stream *stream = cookie;
	struct timeval start;
	int ms;
	int res;
	int written;
	int remaining;

	if (!size) {
		/* You asked to write no data you wrote no data. */
		return 0;
	}

	if (!stream || stream->fd == -1) {
		errno = EBADF;
		return -1;
	}

	if (stream->start.tv_sec) {
		start = stream->start;
	} else {
		start = ast_tvnow();
	}

	written = 0;
	remaining = size;
	for (;;) {
		res = write(stream->fd, buf + written, remaining);
		if (res == remaining) {
			/* Yay everything was written. */
			return size;
		}
		if (0 < res) {
			/* Successfully wrote part of the buffer.  Try to write the rest. */
			written += res;
			remaining -= res;
			continue;
		}
		if (errno != EINTR && errno != EAGAIN) {
			/* Not a retryable error. */
			ast_debug(1, "TCP socket error writing: %s\n", strerror(errno));
			if (written) {
				return written;
			}
			return -1;
		}
		ms = ast_remaining_ms(start, stream->timeout);
		if (!ms) {
			/* Report partial write. */
			ast_debug(1, "TCP timeout writing data\n");
			return written;
		}
		ast_wait_for_output(stream->fd, ms);
	}
}

static void bfcp_tcptls_stream_dtor(void *cookie)
{
#ifdef AST_DEVMODE
	/* Since the ast_assert below is the only one using stream,
	 * and ast_assert is only available with AST_DEVMODE, we
	 * put this in a conditional to avoid compiler warnings. */
	struct ast_tcptls_stream *stream = cookie;
#endif

	ast_assert(stream->fd == -1);
}

static struct ast_tcptls_stream *bfcp_tcptls_stream_alloc(void)
{
	struct ast_tcptls_stream *stream;

	stream = ao2_alloc_options(sizeof(*stream), bfcp_tcptls_stream_dtor,
		AO2_ALLOC_OPT_LOCK_NOLOCK);
	if (stream) {
		stream->fd = -1;
		stream->timeout = -1;
	}
	return stream;
}

static int bfcp_tcptls_stream_close(void *cookie)
{
	struct ast_tcptls_stream *stream = cookie;

	if (!stream) {
		errno = EBADF;
		return -1;
	}

	if (stream->fd != -1) {
		/*
		 * Issuing shutdown() is necessary here to avoid a race
		 * condition where the last data written may not appear
		 * in the TCP stream.  See ASTERISK-23548
		 */
		shutdown(stream->fd, SHUT_RDWR);
		if (close(stream->fd)) {
			ast_log(LOG_ERROR, "close() failed: %s\n", strerror(errno));
		}
		stream->fd = -1;
	}
	ao2_t_ref(stream, -1, "Closed tcptls stream cookie");

	return 0;
}

static FILE *bfcp_tcptls_stream_fopen(struct ast_tcptls_stream *stream, SSL *ssl, int fd, int timeout)
{
	FILE *fp;

#if defined(HAVE_FOPENCOOKIE)	/* the glibc/linux interface */
	static const cookie_io_functions_t cookie_funcs = {
		bfcp_tcptls_stream_read,
		bfcp_tcptls_stream_write,
		NULL,
		bfcp_tcptls_stream_close
	};
#endif	/* defined(HAVE_FOPENCOOKIE) */

	if (fd == -1) {
		/* Socket not open. */
		return NULL;
	}

	stream->ssl = ssl;
	stream->fd = fd;
	stream->timeout = timeout;
	ao2_t_ref(stream, +1, "Opening tcptls stream cookie");

#if defined(HAVE_FUNOPEN)	/* the BSD interface */
	fp = funopen(stream, bfcp_tcptls_stream_read, bfcp_tcptls_stream_write, NULL,
		bfcp_tcptls_stream_close);
#elif defined(HAVE_FOPENCOOKIE)	/* the glibc/linux interface */
	fp = fopencookie(stream, "w+", cookie_funcs);
#else
	/* could add other methods here */
	ast_debug(2, "No stream FILE methods attempted!\n");
	fp = NULL;
#endif

	if (!fp) {
		stream->fd = -1;
		ao2_t_ref(stream, -1, "Failed to open tcptls stream cookie");
	}
	return fp;
}

void ast_bfcp_tcptls_server_start(struct ast_bfcp_tcp *bfcp_tcp)
{
	int flags;
	int x = 1;
	struct ast_tcptls_session_args *desc = &bfcp_tcp->bfcp_tcp_desc;

	int bfcp_tcp_start = ast_get_bfcp_tcp_start();
	int bfcp_tcp_end = ast_get_bfcp_tcp_end();
	/* Do nothing if nothing has changed */
	if (!ast_sockaddr_cmp(&desc->old_address, &desc->local_address)) {
		ast_debug(1, "Nothing changed bfcp tcp in %s\n", desc->name);
		return;
	}

	/* If we return early, there is no one listening */
	ast_sockaddr_setnull(&desc->old_address);

	/* Shutdown a running server if there is one */
	if (desc->master != AST_PTHREADT_NULL) {
		pthread_cancel(desc->master);
		pthread_kill(desc->master, SIGURG);
		pthread_join(desc->master, NULL);
	}

	if (desc->accept_fd != -1) {
		close(desc->accept_fd);
	}

	/* If there's no new server, stop here */
	if (ast_sockaddr_isnull(&desc->local_address)) {
		ast_debug(2, "Server disabled:  %s\n", desc->name);
		return;
	}

	desc->accept_fd = socket(ast_sockaddr_is_ipv6(&desc->local_address) ?
				 AF_INET6 : AF_INET, SOCK_STREAM, 0);
	if (desc->accept_fd < 0) {
		ast_log(LOG_ERROR, "Unable to allocate socket for %s: %s\n", desc->name, strerror(errno));
		return;
	}

	ast_log(LOG_DEBUG, "Bfcp tcptls server start socket fd:%d", desc->accept_fd);

	setsockopt(desc->accept_fd, SOL_SOCKET, SO_REUSEADDR, &x, sizeof(x));

	int bfcp_tcp_port = (bfcp_tcp_start == bfcp_tcp_end) ? bfcp_tcp_start : (ast_random() % (bfcp_tcp_end - bfcp_tcp_start)) + bfcp_tcp_start;
	int startplace = bfcp_tcp_port;
	int cnt = 0;
	for (;;) {
		ast_sockaddr_set_port(&desc->local_address, bfcp_tcp_port);
		if (ast_bind(desc->accept_fd, &desc->local_address) == 0) {
			break;
		}
		if (errno != EADDRINUSE) {
			ast_log(LOG_WARNING, "Unexpected bfcp tcp bind error: %s\n", strerror(errno));
			goto error;
		}

		++bfcp_tcp_port;
		cnt++;

		if (bfcp_tcp_port > bfcp_tcp_end)
			bfcp_tcp_port = bfcp_tcp_start;
		if (bfcp_tcp_port == startplace) {
			ast_log(LOG_WARNING, "No bfcp tcp ports remaining\n");
			goto error;
		}
		if (cnt > 10) {
			ast_log(LOG_ERROR, "Could't allocate a port for bfcp tcp\n");
			goto error;
		}
	}

	if (listen(desc->accept_fd, 10)) {
		ast_log(LOG_ERROR, "Unable to listen for %s!\n", desc->name);
		goto error;
	}
	flags = fcntl(desc->accept_fd, F_GETFL);
	fcntl(desc->accept_fd, F_SETFL, flags | O_NONBLOCK);
	if (ast_pthread_create_background(&desc->master, NULL, desc->accept_fn, bfcp_tcp)) {
		ast_log(LOG_ERROR, "Unable to launch thread for %s on %s: %s\n",
			desc->name,
			ast_sockaddr_stringify(&desc->local_address),
			strerror(errno));
		goto error;
	}

	/* Set current info */
	ast_sockaddr_copy(&desc->old_address, &desc->local_address);

	return;

error:
	close(desc->accept_fd);
	desc->accept_fd = -1;
}

void *handle_bfcp_tcptls_connection(void *data)
{
	struct ast_bfcp_tcp *bfcp_tcp = data;
	struct ast_tcptls_session_instance *tcptls_session = bfcp_tcp->tcptls_session;

	/* TCP/TLS connections are associated with external protocols, and
	 * should not be allowed to execute 'dangerous' functions. This may
	 * need to be pushed down into the individual protocol handlers, but
	 * this seems like a good general policy.
	 */
	if (ast_thread_inhibit_escalations()) {
		ast_log(LOG_ERROR, "Failed to inhibit privilege escalations; killing connection\n");
		ast_tcptls_close_session_file(tcptls_session);
		ao2_ref(tcptls_session, -1);
		return NULL;
	}

	tcptls_session->stream_cookie = bfcp_tcptls_stream_alloc();
	if (!tcptls_session->stream_cookie) {
		ast_tcptls_close_session_file(tcptls_session);
		ao2_ref(tcptls_session, -1);
		return NULL;
	}

	/*
	* open a FILE * as appropriate.
	*/
	if (NULL == tcptls_session->parent->tls_cfg) {
		tcptls_session->f = bfcp_tcptls_stream_fopen(tcptls_session->stream_cookie, NULL,
			tcptls_session->fd, -1);
		if (tcptls_session->f) {
			if (setvbuf(tcptls_session->f, NULL, _IONBF, 0)) {
				ast_tcptls_close_session_file(tcptls_session);
			}
		}
	}

	if (!tcptls_session->f) {
		ast_tcptls_close_session_file(tcptls_session);
		ast_log(LOG_WARNING, "FILE * open failed!\n");
		ao2_ref(tcptls_session, -1);
		return NULL;
	}

	if (tcptls_session->parent->worker_fn) {
		return tcptls_session->parent->worker_fn(bfcp_tcp);
	} else {
		return tcptls_session;
	}
}

void *ast_bfcp_tcptls_server_root(void *data)
{
	struct ast_bfcp_tcp *bfcp_tcp = data;
	struct ast_tcptls_session_args *desc = &bfcp_tcp->bfcp_tcp_desc;
	int fd;
	struct ast_sockaddr addr;
	struct ast_tcptls_session_instance *tcptls_session;
	pthread_t launched;

	for (;;) {
		int i, flags;

		if (desc->periodic_fn) {
			desc->periodic_fn(desc);
		}
		i = ast_wait_for_input(desc->accept_fd, desc->poll_timeout);
		if (i <= 0) {
			continue;
		}
		fd = ast_accept(desc->accept_fd, &addr);
		if (fd < 0) {
			if ((errno != EAGAIN) && (errno != EWOULDBLOCK) && (errno != EINTR) && (errno != ECONNABORTED)) {
				ast_log(LOG_ERROR, "Accept failed: %s\n", strerror(errno));
				break;
			}
			continue;
		}
		ast_log(LOG_DEBUG, "Bfcp tcptls server root accept fd:%d", fd);

		tcptls_session = ao2_alloc(sizeof(*tcptls_session), bfcp_session_instance_destructor);
		if (!tcptls_session) {
			ast_log(LOG_WARNING, "No memory for new bfcp tcp session: %s\n", strerror(errno));
			if (close(fd)) {
				ast_log(LOG_ERROR, "close() bfcp tcp failed: %s\n", strerror(errno));
			}
			continue;
		}

		tcptls_session->overflow_buf = ast_str_create(128);
		flags = fcntl(fd, F_GETFL);
		fcntl(fd, F_SETFL, flags & ~O_NONBLOCK);
		tcptls_session->fd = fd;
		tcptls_session->parent = desc;
		ast_sockaddr_copy(&tcptls_session->remote_address, &addr);
		tcptls_session->client = 0;
		bfcp_tcp->tcptls_session = tcptls_session;
		bfcp_tcp->destoryed_tcptls_session = 0;

		ast_log(LOG_DEBUG,"ast_bfcp_tcptls_server_root tcptls_session:%p", tcptls_session);

		/* This thread is now the only place that controls the single ref to tcptls_session */
		if (ast_pthread_create_detached_background(&launched, NULL, handle_bfcp_tcptls_connection, bfcp_tcp)) {
			ast_log(LOG_ERROR, "Unable to launch helper thread: %s\n", strerror(errno));
			ast_tcptls_close_session_file(tcptls_session);
			ao2_ref(tcptls_session, -1);
		}
	}

	return NULL;
}


struct ast_tcptls_session_instance *ast_bfcp_tcptls_client_start(struct ast_bfcp_tcp *bfcp_tcp)
{
	struct ast_tcptls_session_instance *tcptls_session = bfcp_tcp->tcptls_session;
	struct ast_tcptls_session_args *desc;
	int flags;

	if (!(desc = tcptls_session->parent)) {
		goto client_start_error;
	}
	ast_log(LOG_DEBUG, "ast_bfcp_tcptls_client_start connnect");
	if (ast_connect(desc->accept_fd, &desc->remote_address)) {
		ast_log(LOG_ERROR, "Unable to connect %s to %s: %s\n",
			desc->name,
			ast_sockaddr_stringify(&desc->remote_address),
			strerror(errno));
		goto client_start_error;
	}

	flags = fcntl(desc->accept_fd, F_GETFL);
	fcntl(desc->accept_fd, F_SETFL, flags & ~O_NONBLOCK);

	return handle_bfcp_tcptls_connection(bfcp_tcp);

client_start_error:
	if (desc) {
		close(desc->accept_fd);
		desc->accept_fd = -1;
	}
	ao2_ref(tcptls_session, -1);
	return NULL;

}

struct ast_tcptls_session_instance *ast_bfcp_tcptls_client_create(struct ast_tcptls_session_args *desc)
{
	ast_log(LOG_DEBUG, "ast_bfcp_tcptls_client_create begin");
	int x = 1;
	struct ast_tcptls_session_instance *tcptls_session = NULL;

	int bfcp_tcp_start = ast_get_bfcp_tcp_start();
	int bfcp_tcp_end = ast_get_bfcp_tcp_end();
	/* Do nothing if nothing has changed */
	if (!ast_sockaddr_cmp(&desc->old_address, &desc->remote_address)) {
		ast_debug(1, "Nothing changed in %s\n", desc->name);
		return NULL;
	}

	/* If we return early, there is no connection */
	ast_sockaddr_setnull(&desc->old_address);

	if (desc->accept_fd != -1) {
		close(desc->accept_fd);
	}

	desc->accept_fd = socket(ast_sockaddr_is_ipv6(&desc->remote_address) ?
				 AF_INET6 : AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (desc->accept_fd < 0) {
		ast_log(LOG_ERROR, "Unable to allocate socket for %s: %s\n",
			desc->name, strerror(errno));
		return NULL;
	}

	ast_log(LOG_DEBUG, "ast_bfcp_tcptls_client_create fd:%d", desc->accept_fd);

	/* if a local address was specified, bind to it so the connection will
	   originate from the desired address */
	if (!ast_sockaddr_isnull(&desc->local_address)) {
		setsockopt(desc->accept_fd, SOL_SOCKET, SO_REUSEADDR, &x, sizeof(x));

		int bfcp_tcp_port = (bfcp_tcp_start == bfcp_tcp_end) ? bfcp_tcp_start : (ast_random() % (bfcp_tcp_end - bfcp_tcp_start)) + bfcp_tcp_start;
		int startplace = bfcp_tcp_port;
		int cnt = 0;
		for (;;) {
			ast_sockaddr_set_port(&desc->local_address, bfcp_tcp_port);
			if (ast_bind(desc->accept_fd, &desc->local_address) == 0) {
				break;
			}
			if (errno != EADDRINUSE) {
				ast_log(LOG_WARNING, "Unexpected bfcp bind error: %s\n", strerror(errno));
				goto error;
			}

			++bfcp_tcp_port;
			cnt++;

			if (bfcp_tcp_port > bfcp_tcp_end)
				bfcp_tcp_port = bfcp_tcp_start;
			if (bfcp_tcp_port == startplace) {
				ast_log(LOG_WARNING, "No bfcp ports remaining\n");
				goto error;
			}
			if (cnt > 10) {
				ast_log(LOG_ERROR, "Could't allocate a port for bfcp\n");
				goto error;
			}
		}
	}

	if (!(tcptls_session = ao2_alloc(sizeof(*tcptls_session), bfcp_session_instance_destructor))) {
		goto error;
	}

	tcptls_session->overflow_buf = ast_str_create(128);
	tcptls_session->client = 1;
	tcptls_session->fd = desc->accept_fd;
	tcptls_session->parent = desc;
	tcptls_session->parent->worker_fn = NULL;
	ast_sockaddr_copy(&tcptls_session->remote_address, &desc->remote_address);

	/* Set current info */
	ast_sockaddr_copy(&desc->old_address, &desc->remote_address);
	ast_log(LOG_DEBUG,"ast_bfcp_tcptls_client_create tcptls_session:%p", tcptls_session);
	return tcptls_session;

error:
	close(desc->accept_fd);
	desc->accept_fd = -1;
	if (tcptls_session) {
		ao2_ref(tcptls_session, -1);
	}
	return NULL;
}

HOOK_T ast_bfcp_tcptls_server_read(struct ast_tcptls_session_instance *tcptls_session, void *buf, size_t count)
{
	if (!tcptls_session) {
		ast_log(LOG_ERROR, "TCP/TLS read called on invalid session.\n");
		errno = EIO;
		return -1;
	}
	if (tcptls_session->fd == -1) {
		ast_log(LOG_ERROR, "TCP/TLS read called on invalid fd.\n");
		errno = EIO;
		return -1;
	}
	if (tcptls_session->f == NULL) {
		ast_log(LOG_ERROR, "TCP/TLS read called on invalid file.\n");
		errno = EIO;
		return -1;
	}
	if (!tcptls_session->stream_cookie || tcptls_session->stream_cookie->fd == -1) {
		ast_log(LOG_ERROR, "TCP/TLS read called on invalid stream.\n");
		errno = EIO;
		return -1;
	}

	return bfcp_tcptls_stream_read(tcptls_session->stream_cookie, buf, count);
}

HOOK_T ast_bfcp_tcptls_server_write(struct ast_tcptls_session_instance *tcptls_session, const void *buf, size_t count)
{
	if (!tcptls_session) {
		ast_log(LOG_ERROR, "TCP/TLS write called on invalid session.\n");
		errno = EIO;
		return -1;
	}
	if (tcptls_session->fd == -1) {
		ast_log(LOG_ERROR, "TCP/TLS write called on invalid fd.\n");
		errno = EIO;
		return -1;
	}
	if (tcptls_session->f == NULL) {
		ast_log(LOG_ERROR, "TCP/TLS write called on invalid file.\n");
		errno = EIO;
		return -1;
	}
	if (!tcptls_session->stream_cookie || tcptls_session->stream_cookie->fd == -1) {
		ast_log(LOG_ERROR, "TCP/TLS write called on invalid stream.\n");
		errno = EIO;
		return -1;
	}

	return bfcp_tcptls_stream_write(tcptls_session->stream_cookie, buf, count);
}

void bfcp_init_tcp_desc(struct ast_tcptls_session_args *bfcp_tcp_desc)
{
	bfcp_tcp_desc->accept_fd = -1;
	bfcp_tcp_desc->master = AST_PTHREADT_NULL;
	bfcp_tcp_desc->tls_cfg = NULL;
	bfcp_tcp_desc->poll_timeout = -1;
	bfcp_tcp_desc->name = "BFCP TCP server";
	bfcp_tcp_desc->accept_fn = ast_bfcp_tcptls_server_root;
	bfcp_tcp_desc->worker_fn = bfcp_tcp_worker_fn;

	return;
}

struct ast_bfcp_tcp *ast_tcp_bfcp_new_with_bindaddr(struct ast_sockaddr *addr)
{
	struct ast_bfcp_tcp *bfcp_tcp;

	ast_log(LOG_DEBUG, "ast_bfcp_new_with_bindaddr begin\n");

	if (!(bfcp_tcp = ast_calloc(1, sizeof(*bfcp_tcp)))) {
		ast_log(LOG_WARNING, "Unable to alloc bfcp tcp\n");
		return NULL;
	}

	/* Start TCP server */
	bfcp_init_tcp_desc(&bfcp_tcp->bfcp_tcp_desc);
	if (ast_sockaddr_isnull(&bfcp_tcp->bfcp_tcp_desc.local_address)) {
		ast_sockaddr_copy(&bfcp_tcp->bfcp_tcp_desc.local_address, addr);
	}

	ast_bfcp_tcptls_server_start(bfcp_tcp);
	if (bfcp_tcp->bfcp_tcp_desc.accept_fd == -1) {
		/* TCP server start failed. Tell the admin */
		ast_log(LOG_ERROR, "BFCP TCP Server start failed. Not listening on TCP socket.\n");
	} else {
		ast_log(LOG_DEBUG, "BFCP TCP server started\n");
	}

	return bfcp_tcp;
}

struct ast_bfcp_tcp *create_peer_bfcp_tcp(struct ast_sockaddr *addr)
{	
	struct ast_bfcp_tcp *bfcp_tcp;

	ast_log(LOG_DEBUG, "create_peer_bfcp_tcp begin\n");

	if (!(bfcp_tcp = ast_calloc(1, sizeof(*bfcp_tcp)))) {
		ast_log(LOG_WARNING, "Unable to alloc bfcp tcp\n");
		return NULL;
	}
	ast_verbose("create_peer_bfcp_tcp peer remote BFCP is at %s\n", ast_sockaddr_stringify(addr));
	ast_sockaddr_copy(&bfcp_tcp->peer_remote_address, addr);
	return bfcp_tcp;
}


/*! \todo Get socket for dialog, prepare if needed, and return file handle  */
int bfcp_prepare_socket(struct ast_bfcp_tcp *bfcp_tcp, struct ast_bfcp_tcp *peer_bfcp_tcp)
{
	ast_log(LOG_DEBUG, "bfcp_prepare_socket begin");
	static const char name[] = "BFCP TCP client";
	struct ast_tcptls_session_instance *tcptls_session;
	struct ast_tcptls_session_args *client_args;
	pthread_t launched;

	if (peer_bfcp_tcp && peer_bfcp_tcp->destoryed_tcptls_session) {
		ast_log(LOG_WARNING, "bfcp tcp is destoryed tcptls session now");
		return -1;
	}

	if (NULL == peer_bfcp_tcp) {
		peer_bfcp_tcp = create_peer_bfcp_tcp(&bfcp_tcp->remote_address);
		if (NULL == peer_bfcp_tcp) {
			ast_log(LOG_ERROR, "Can't creat peer bfcp tcp");
			return -1;
		}
		bfcp_tcp->peer_bfcp_tcp = peer_bfcp_tcp;
		peer_bfcp_tcp->peer_bfcp_tcp = bfcp_tcp;
	}


	if ((peer_bfcp_tcp->tcptls_session) &&
		(peer_bfcp_tcp->tcptls_session->fd != -1)) {
		ast_log(LOG_DEBUG, "Bfcp tcp client already have fd now:%p %d", peer_bfcp_tcp->tcptls_session, peer_bfcp_tcp->tcptls_session->fd);
		return peer_bfcp_tcp->tcptls_session->fd;
	}

	/* create new session arguments for the client connection */
	if (!(client_args = ao2_alloc(sizeof(*client_args), bfcp_tcptls_client_args_destructor)) ||
		!(client_args->name = ast_strdup(name))) {
		goto create_tcptls_session_fail;
	}

	client_args->accept_fd = -1;
	ast_sockaddr_copy(&client_args->remote_address, &bfcp_tcp->peer_remote_address);
	ast_log(LOG_DEBUG, "BFCP old remote at port %s\n", ast_sockaddr_stringify(&client_args->old_address));
	ast_log(LOG_DEBUG, "BFCP peer remote at port %s\n", ast_sockaddr_stringify(&bfcp_tcp->peer_remote_address));
	/* Create a client connection for address, this does not start the connection, just sets it up. */
	if (!(peer_bfcp_tcp->tcptls_session = ast_bfcp_tcptls_client_create(client_args))) {
		goto create_tcptls_session_fail;
	}
	peer_bfcp_tcp->destoryed_tcptls_session = 0;
	if (!(tcptls_session = ast_bfcp_tcptls_client_start(peer_bfcp_tcp))) {
		ast_log(LOG_DEBUG, "bfcp tcp client start failed\n");
		goto create_tcptls_session_fail;
	}

	if (ast_pthread_create_detached_background(&launched, NULL, bfcp_tcp_worker_fn, peer_bfcp_tcp)) {
		ast_debug(1, "Unable to launch bfcp tcp '%s'.", client_args->name);
		ao2_ref(peer_bfcp_tcp->tcptls_session, -1); /* take away the thread ref we just gave it */
		goto create_tcptls_session_fail;
	}

	return peer_bfcp_tcp->tcptls_session->fd;

create_tcptls_session_fail:
	ast_log(LOG_DEBUG, "bfcp_prepare_socket cleanup tcptls_session begin");
	if (client_args) {
		ao2_t_ref(client_args, -1, "failed to create client, getting rid of client tcptls_session arguments");
	}
	if (peer_bfcp_tcp->tcptls_session && (0 == peer_bfcp_tcp->destoryed_tcptls_session)) {
		peer_bfcp_tcp->destoryed_tcptls_session = 1;
		ast_log(LOG_DEBUG,"bfcp_prepare_socket cleanup:%p", peer_bfcp_tcp->tcptls_session);
		ao2_lock(peer_bfcp_tcp->tcptls_session);
		ast_tcptls_close_session_file(peer_bfcp_tcp->tcptls_session);
		peer_bfcp_tcp->tcptls_session->parent = NULL;
		ao2_unlock(peer_bfcp_tcp->tcptls_session);

		ao2_ref(peer_bfcp_tcp->tcptls_session, -1);
		peer_bfcp_tcp->tcptls_session = NULL;
	}

	return -1;
}

void bfcp_tcptls_client_args_destructor(void *obj)
{
	struct ast_tcptls_session_args *args = obj;
	ast_log(LOG_DEBUG, "bfcp_tcptls_client_args_destructor begin:%p", args);
	if (args->tls_cfg) {
		ast_free(args->tls_cfg->certfile);
		ast_free(args->tls_cfg->pvtfile);
		ast_free(args->tls_cfg->cipher);
		ast_free(args->tls_cfg->cafile);
		ast_free(args->tls_cfg->capath);

		ast_ssl_teardown(args->tls_cfg);
	}
	ast_free(args->tls_cfg);
	ast_free((char *) args->name);
}

/*! \brief Check if the authtimeout has expired.
 * \param start the time when the session started
 *
 * \retval 0 the timeout has expired
 * \retval -1 error
 * \return the number of milliseconds until the timeout will expire
 */
int bfcp_check_authtimeout(time_t start)
{
	int timeout;
	time_t now;
	if(time(&now) == -1) {
		ast_log(LOG_ERROR, "error executing time(): %s\n", strerror(errno));
		return -1;
	}

	timeout = (bfcp_authtimeout - (now - start)) * 1000;
	if (timeout < 0) {
		/* we have timed out */
		return 0;
	}

	return timeout;
}

void *_bfcp_tcp_helper_thread(struct ast_bfcp_tcp *bfcp_tcp)
{
	int res, timeout = -1, authenticated = 0, flags;
	time_t start;
	struct pollfd fds[1];
	memset(fds, 0, sizeof(fds));
	struct ast_tcptls_session_args *client_args = NULL;

	ast_log(LOG_DEBUG, "Bfcp tcp helper thread begin");

	struct ast_tcptls_session_instance *tcptls_session = bfcp_tcp->tcptls_session;

	if (!tcptls_session) {
		ast_log(LOG_DEBUG, "Bfcp tcp helper thread tcptls session is NULL");
		return NULL;
	}

	if (bfcp_tcp->peer_bfcp_tcp && bfcp_tcp->peer_bfcp_tcp->destoryed_tcptls_session) {
		ast_log(LOG_WARNING, "bfcp tcp is destorying tcptls session now");
		goto cleanup;
	}

	if (!tcptls_session->client) {
		if (ast_atomic_fetchadd_int(&bfcp_unauth_sessions, +1) >= bfcp_authlimit) {
			/* unauth_sessions is decremented in the cleanup code */
			ast_log(LOG_ERROR, "unauth_sessions is decremented in the cleanup code");
			goto cleanup;
		}

		if ((flags = fcntl(tcptls_session->fd, F_GETFL)) == -1) {
			ast_log(LOG_ERROR, "error setting socket to non blocking mode, fcntl() failed: %s\n", strerror(errno));
			goto cleanup;
		}

		flags |= O_NONBLOCK;
		if (fcntl(tcptls_session->fd, F_SETFL, flags) == -1) {
			ast_log(LOG_ERROR, "error setting socket to non blocking mode, fcntl() failed: %s\n", strerror(errno));
			goto cleanup;
		}
	} else {
		if (!(client_args = tcptls_session->parent)) {
			goto cleanup;
		}
	}

	flags = 1;
	if (setsockopt(tcptls_session->fd, SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
		ast_log(LOG_ERROR, "Error enabling TCP keep-alives on bfcp socket: %s\n", strerror(errno));
		goto cleanup;
	}

	/* set up pollfd to watch for reads on the socket */
	fds[0].fd = tcptls_session->fd;
	fds[0].events = POLLIN | POLLPRI;

	if(time(&start) == -1) {
		ast_log(LOG_ERROR, "Error executing time(): %s\n", strerror(errno));
		goto cleanup;
	}

	/*
	 * We cannot let the stream exclusively wait for data to arrive.
	 * We have to wake up the task to send outgoing messages.
	 */
	ast_tcptls_stream_set_exclusive_input(tcptls_session->stream_cookie, 0);

	ast_tcptls_stream_set_timeout_sequence(tcptls_session->stream_cookie, ast_tvnow(),
		tcptls_session->client ? -1 : (bfcp_authtimeout * 1000));

	for (;;) {
		if (bfcp_tcp->peer_bfcp_tcp && bfcp_tcp->peer_bfcp_tcp->destoryed_tcptls_session) {
			ast_log(LOG_WARNING, "bfcp tcp is destorying tcptls session now");
			goto cleanup;
		}

		if (!tcptls_session->client && !authenticated) {
			authenticated = 1;
			ast_tcptls_stream_set_timeout_disable(tcptls_session->stream_cookie);
			ast_atomic_fetchadd_int(&bfcp_unauth_sessions, -1);
		}

		/* calculate the timeout for unauthenticated server sessions */
		if (!tcptls_session->client && !authenticated ) {
			if ((timeout = bfcp_check_authtimeout(start)) < 0) {
				goto cleanup;
			}

			if (timeout == 0) {
				ast_debug(2, "BFCP TCP %s timed out\n", tcptls_session->client ? "client" : "server");
				goto cleanup;
			}
		} else {
			timeout = -1;
		}

		res = ast_poll(fds, 1, timeout); /* poll for bfcp tcp socket */
		if (res < 0) {
			ast_debug(2, "BFCP TCP %s :: ast_wait_for_input returned %d\n", tcptls_session->client ? "client" : "server", res);
			goto cleanup;
		} else if (res == 0) {
			/* timeout */
			ast_debug(2, "BFCP TCP %s timed out\n", tcptls_session->client ? "client" : "server");
			goto cleanup;
		}

		if (fds[0].revents) { /* there is data on the socket to be read */
			fds[0].revents = 0;

			char readbuf[4097];
			memset(readbuf, 0, sizeof(readbuf));
			res = ast_bfcp_tcptls_server_read(tcptls_session, readbuf, sizeof(readbuf) - 1);
			if (res <= 0) {
				ast_log(LOG_DEBUG, "Bfcp read no data");
				goto cleanup;
			}

			ast_log(LOG_DEBUG, "Bfcp read socket data len:%d, %p", res, bfcp_tcp->tcptls_session);

			if (bfcp_prepare_socket(bfcp_tcp, bfcp_tcp->peer_bfcp_tcp) < 0) {
				ast_log(LOG_ERROR, "Bfcp prepare socket error");
				goto cleanup;
			}
			ao2_lock(bfcp_tcp->peer_bfcp_tcp->tcptls_session);
			ast_bfcp_tcptls_server_write(bfcp_tcp->peer_bfcp_tcp->tcptls_session, readbuf, res);
			ao2_unlock(bfcp_tcp->peer_bfcp_tcp->tcptls_session);
		}
	}

	ast_debug(2, "Shutting down thread for BFCP TCP %s\n", tcptls_session->client ? "client" : "server");

cleanup:
	ast_log(LOG_DEBUG, "Bfcp tcp helper thread cleanup");

	if (tcptls_session && !tcptls_session->client && !authenticated) {
		ast_atomic_fetchadd_int(&bfcp_unauth_sessions, -1);
	}

	/* if client, we own the parent session arguments and must decrement ref */
	if (client_args) {
		ao2_t_ref(client_args, -1, "closing tcptls thread, getting rid of client tcptls_session arguments");
	}

	if (tcptls_session && (0 == bfcp_tcp->destoryed_tcptls_session)) {
		ast_log(LOG_DEBUG,"tcptls_session cleanup:%p", tcptls_session);
		bfcp_tcp->destoryed_tcptls_session = 1;
		ao2_lock(tcptls_session);
		ast_tcptls_close_session_file(tcptls_session);
		tcptls_session->parent = NULL;
		ao2_unlock(tcptls_session);

		ao2_ref(tcptls_session, -1);
		tcptls_session = NULL;
	}

	return NULL;
}

void *bfcp_tcp_worker_fn(void *data)
{
	struct ast_bfcp_tcp *bfcp_tcp = data;

	return _bfcp_tcp_helper_thread(bfcp_tcp);
}

void ast_bfcp_tcptls_server_stop(struct ast_bfcp_tcp *bfcp_tcp)
{
	if (NULL == bfcp_tcp) {
		ast_log(LOG_WARNING, "bfcp tcp stop, has no bfcp\n");
		return;
	}
	struct ast_tcptls_session_args *desc = &bfcp_tcp->bfcp_tcp_desc;
	if (NULL == desc) {
		ast_log(LOG_WARNING, "bfcp tcp stop, has no desc\n");
		return;
	}
	if (desc->master != AST_PTHREADT_NULL) {
		pthread_cancel(desc->master);
		pthread_kill(desc->master, SIGURG);
		pthread_join(desc->master, NULL);
		desc->master = AST_PTHREADT_NULL;
	}
	if (desc->accept_fd != -1) {
		close(desc->accept_fd);
	}
	desc->accept_fd = -1;
	ast_debug(2, "Stopped server :: %s\n", desc->name);
}

void ast_bfcp_tcp_destroy(struct ast_bfcp_tcp *bfcp_tcp)
{
	ast_log(LOG_DEBUG, "ast_bfcp_tcp_destroy begin");

	if (NULL == bfcp_tcp) {
		ast_log(LOG_WARNING, "bfcp is NULL, destory failed");
		return;
	}
	ast_bfcp_tcptls_server_stop(bfcp_tcp);

	ast_free(bfcp_tcp);

	return;
}

#endif

