/*
 * Vinetic-SVIP resource manager.
 *
 * Copyright (C) 2012 - 2014, Grandstream Networks, Inc.
 *
 * jwli@grandstream.com
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */


/*!
 * \file
 * \brief Vinetic-SVIP resource manager
 *
 * \author Javen <jwli@grandstream.com>
 */

/*** MODULEINFO
	<support_level>core</support_level>
 ***/

#include "asterisk.h"
#include "asterisk/lock.h"
#include "asterisk/svipmanager.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 20141012 $");

static struct svip_dev_cod_res g_svip_res[MAX_VINETIC_DEV_NUM];
static ast_mutex_t svip_res_lock;

int ast_get_svip_codec(unsigned int dev, cod_ru_type_t ru_type)
{
	int i;

	if (dev >= MAX_VINETIC_DEV_NUM || dev < 0)
		return -1;

	ast_mutex_lock(&svip_res_lock);
	for (i=0; i<g_svip_res[dev].ru_num; i++) {
		if ((ru_type == g_svip_res[dev].cod_ru[i].ru_type) &&
			(0 == g_svip_res[dev].cod_ru[i].inused)) {
			g_svip_res[dev].cod_ru[i].inused = 1;
			ast_mutex_unlock(&svip_res_lock);
			return i;
		}
	}

	if (COD_G711 == ru_type) {
		for (i=0; i<g_svip_res[dev].ru_num; i++) {
			if ((COD_LBR == g_svip_res[dev].cod_ru[i].ru_type) &&
				(0 == g_svip_res[dev].cod_ru[i].inused)) {
				g_svip_res[dev].cod_ru[i].inused = 1;
				ast_mutex_unlock(&svip_res_lock);
				return i;
			}
		}
	}
	ast_mutex_unlock(&svip_res_lock);

	return -1;
}

int ast_choose_optimal_dev(void)
{
	int ndev, nchan;
	int lbr_idle;
	int lbr_idle_max = 0;
	int lbr_idle_dev = 0;

	ast_mutex_lock(&svip_res_lock);
	for (ndev=0; ndev<MAX_VINETIC_DEV_NUM; ndev++) {
		if (0 == g_svip_res[ndev].ru_num)
			break;

		lbr_idle = 0;
		for (nchan=0; nchan<g_svip_res[ndev].ru_num; nchan++) {
			if ((COD_LBR == g_svip_res[ndev].cod_ru[nchan].ru_type) &&
				(0 == g_svip_res[ndev].cod_ru[nchan].inused))
				lbr_idle ++;
		}

		if (lbr_idle >= lbr_idle_max) {
			lbr_idle_max = lbr_idle;
			lbr_idle_dev = ndev;
		}
	}
	ast_mutex_unlock(&svip_res_lock);

	return lbr_idle_dev;
}

int ast_put_svip_codec(unsigned int dev, unsigned int chan)
{
	if (dev >= MAX_VINETIC_DEV_NUM || dev < 0)
		return -1;

	if (chan >= MAX_VINETIC_CHAN_PER_DEV || chan < 0)
		return -1;

	ast_mutex_lock(&svip_res_lock);
	g_svip_res[dev].cod_ru[chan].inused = 0;
	ast_mutex_unlock(&svip_res_lock);

	return 0;
}

int ast_svip_register(unsigned int dev, unsigned int channum)
{
	int i;

	if (dev >= MAX_VINETIC_DEV_NUM || dev < 0)
		return -1;

	if (channum > MAX_VINETIC_CHAN_PER_DEV)
		channum = MAX_VINETIC_CHAN_PER_DEV;

	ast_mutex_lock(&svip_res_lock);
	for (i=0; i<channum/2; i++) {
		g_svip_res[dev].cod_ru[i].ru_type = COD_LBR;
		g_svip_res[dev].cod_ru[i].inused = 0;
	}

	for (i=channum/2; i<channum; i++) {
		g_svip_res[dev].cod_ru[i].ru_type = COD_G711_ONLY;
		g_svip_res[dev].cod_ru[i].inused = 0;
	}
	g_svip_res[dev].ru_num = channum;
	ast_mutex_unlock(&svip_res_lock);

	return 0;
}

int ast_svip_unregister(unsigned int dev)
{
	if (dev >= MAX_VINETIC_DEV_NUM || dev < 0)
		return -1;

	ast_mutex_lock(&svip_res_lock);
	g_svip_res[dev].ru_num = 0;
	ast_mutex_unlock(&svip_res_lock);

	return 0;
}

int ast_svipmanager_init(void)
{
	memset(g_svip_res, 0x0, sizeof(g_svip_res));
	ast_mutex_init(&svip_res_lock);

	return 0;
}
